@@ -1,6 +1,6 @@
 package Net::DNS::RR::MB;
 
-# $Id: MB.pm,v 1.2 1997/06/13 03:33:54 mfuhr Exp $
+# $Id: MB.pm,v 1.3 1997/07/06 16:31:54 mfuhr Exp $
 
 use strict;
 use vars qw(@ISA);
@@ -20,6 +20,17 @@ sub new {
 	return bless $self, $class;
 }
 
+sub new_from_string {
+	my ($class, $self, $string) = @_;
+
+	if ($string) {
+		$string =~ s/\.+$//;
+		$self->{"madname"} = $string;
+	}
+
+	return bless $self, $class;
+}
+
 sub rdatastr {
 	my $self = shift;
 
