@@ -1,6 +1,6 @@
 package Net::DNS::RR::SOA;
 
-# $Id: SOA.pm,v 1.3 1997/06/13 03:33:54 mfuhr Exp $
+# $Id: SOA.pm,v 1.4 1997/07/06 16:31:54 mfuhr Exp $
 
 use strict;
 use vars qw(@ISA);
@@ -32,6 +32,31 @@ sub new {
 	return bless $self, $class;
 }
 
+sub new_from_string {
+	my ($class, $self, $string) = @_;
+
+	if ($string) {
+		$string =~ tr/()//d;
+		$string =~ s/;.*$//mg;
+
+		my ($mname, $rname, $serial,
+		    $refresh, $retry, $expire, $minimum) = $string =~ /(\S+)/g;
+
+		$mname =~ s/\.+$//;
+		$rname =~ s/\.+$//;
+
+		$self->{"mname"}   = $mname;
+		$self->{"rname"}   = $rname;
+		$self->{"serial"}  = $serial;
+		$self->{"refresh"} = $refresh;
+		$self->{"retry"}   = $retry;
+		$self->{"expire"}  = $expire;
+		$self->{"minimum"} = $minimum;
+	}
+
+	return bless $self, $class;
+}
+
 sub rdatastr {
 	my $self = shift;
 	my $rdatastr;
